@@ -959,7 +959,7 @@ run_theme_benchmark (void)
 {
   GtkWidget* widget;
   cairo_surface_t *pixmap;
-  int top_height, bottom_height, left_width, right_width;
+  MetaFrameBorders borders;
   MetaButtonState button_states[META_BUTTON_TYPE_LAST] =
   {
     META_BUTTON_STATE_NORMAL,
@@ -986,10 +986,7 @@ run_theme_benchmark (void)
                                 META_FRAME_TYPE_NORMAL,
                                 get_text_height (widget),
                                 get_flags (widget),
-                                &top_height,
-                                &bottom_height,
-                                &left_width,
-                                &right_width);
+                                &borders);
   
   layout = create_title_layout (widget);
   
@@ -1024,8 +1021,8 @@ run_theme_benchmark (void)
        */
       pixmap = gdk_window_create_similar_surface (gtk_widget_get_window (widget),
                                                   CAIRO_CONTENT_COLOR,
-                                                  client_width + left_width + right_width,
-                                                  client_height + top_height + bottom_height);
+                                                  client_width + borders.visible.left + borders.visible.right,
+                                                  client_height + borders.visible.top + borders.visible.bottom);
 
       cr = cairo_create (pixmap);
 
