@@ -23,10 +23,10 @@
  */
 
 #include <config.h>
-#include "prefs.h"
+#include <meta/prefs.h>
 #include "ui.h"
 #include "frames.h"
-#include "util.h"
+#include <meta/util.h>
 #include "menu.h"
 #include "core.h"
 #include "theme-private.h"
@@ -58,7 +58,7 @@ struct _MetaUI
 };
 
 void
-meta_ui_init (int *argc, char ***argv)
+meta_ui_init (void)
 {
   /* As of 2.91.7, Gdk uses XI2 by default, which conflicts with the
    * direct X calls we use - in particular, events caused by calls to
@@ -70,7 +70,7 @@ meta_ui_init (int *argc, char ***argv)
   gdk_disable_multidevice ();
 #endif
 
-  if (!gtk_init_check (argc, argv))
+  if (!gtk_init_check (NULL, NULL))
     meta_fatal ("Unable to open X display %s\n", XDisplayName (NULL));
 
   meta_stock_icons_init ();
@@ -306,14 +306,25 @@ meta_ui_free (MetaUI *ui)
 }
 
 void
-meta_ui_get_frame_geometry (MetaUI *ui,
-                            Window frame_xwindow,
-                            int *top_height, int *bottom_height,
-                            int *left_width, int *right_width)
+meta_ui_get_frame_borders (MetaUI *ui,
+                           Window frame_xwindow,
+                           MetaFrameBorders *borders)
 {
-  meta_frames_get_geometry (ui->frames, frame_xwindow,
-                            top_height, bottom_height,
-                            left_width, right_width);
+  meta_frames_get_borders (ui->frames, frame_xwindow,
+                           borders);
+}
+
+void
+meta_ui_get_corner_radiuses (MetaUI *ui,
+                             Window  xwindow,
+                             float  *top_left,
+                             float  *top_right,
+                             float  *bottom_left,
+                             float  *bottom_right)
+{
+  meta_frames_get_corner_radiuses (ui->frames, xwindow,
+                                   top_left, top_right,
+                                   bottom_left, bottom_right);
 }
 
 Window
@@ -444,6 +455,13 @@ meta_ui_unflicker_frame_bg (MetaUI *ui,
                             target_width, target_height);
 }
 
+void
+meta_ui_update_frame_style (MetaUI  *ui,
+                            Window   xwindow)
+{
+  meta_frames_update_frame_style (ui->frames, xwindow);
+}
+
 void
 meta_ui_repaint_frame (MetaUI *ui,
                        Window xwindow)
@@ -458,16 +476,14 @@ meta_ui_reset_frame_bg (MetaUI *ui,
   meta_frames_reset_bg (ui->frames, xwindow);
 }
 
-void
-meta_ui_apply_frame_shape  (MetaUI  *ui,
-                            Window   xwindow,
-                            int      new_window_width,
-                            int      new_window_height,
-                            gboolean window_has_shape)
+cairo_region_t *
+meta_ui_get_frame_bounds (MetaUI  *ui,
+                          Window   xwindow,
+                          int      window_width,
+                          int      window_height)
 {
-  meta_frames_apply_shapes (ui->frames, xwindow,
-                            new_window_width, new_window_height,
-                            window_has_shape);
+  return meta_frames_get_frame_bounds (ui->frames, xwindow,
+                                       window_width, window_height);
 }
 
 void
@@ -477,7 +493,6 @@ meta_ui_queue_frame_draw (MetaUI *ui,
   meta_frames_queue_draw (ui->frames, xwindow);
 }
 
-
 void
 meta_ui_set_frame_title (MetaUI     *ui,
                          Window      xwindow,
@@ -707,15 +722,12 @@ void
 meta_ui_theme_get_frame_borders (MetaUI *ui,
                                  MetaFrameType      type,
                                  MetaFrameFlags     flags,
-                                 int               *top_height,
-                                 int               *bottom_height,
-                                 int               *left_width,
-                                 int               *right_width)
+                                 MetaFrameBorders  *borders)
 {
   int text_height;
+  GtkStyleContext *style = NULL;
   PangoContext *context;
   const PangoFontDescription *font_desc;
-  GtkStyle *default_style;
 
   if (meta_ui_have_a_theme ())
     {
@@ -724,21 +736,23 @@ meta_ui_theme_get_frame_borders (MetaUI *ui,
 
       if (!font_desc)
         {
-          default_style = gtk_widget_get_default_style ();
-          font_desc = default_style->font_desc;
+          style = gtk_style_context_new ();
+          font_desc = gtk_style_context_get_font (style, 0);
         }
 
       text_height = meta_pango_font_desc_get_text_height (font_desc, context);
 
       meta_theme_get_frame_borders (meta_theme_get_current (),
                                     type, text_height, flags,
-                                    top_height, bottom_height,
-                                    left_width, right_width);
+                                    borders);
     }
   else
     {
-      *top_height = *bottom_height = *left_width = *right_width = 0;
+      meta_frame_borders_clear (borders);
     }
+
+  if (style != NULL)
+    g_object_unref (style);
 }
 
 void
